// =============================================================================
// Scilab ( https://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2014 - Scilab Enterprises - Paul Bignier
//
//  This file is distributed under the same license as the Scilab package.
// =============================================================================
//
// <-- CLI SHELL MODE -->
// <-- ENGLISH IMPOSED -->
//
// <-- Non-regression test for bug 13243 -->
//
// <-- GitLab URL -->
// https://gitlab.com/scilab/scilab/-/issues/13243
//
// <-- Short Description -->
// optim() with "gc" option failed when iprint<0.

function f = problem1(x)
    f = sum((x-(1:length(x))').^2);
endfunction

// Define the cost function:
function [f, g, ind] = myoptfuncfixedstep(x, ind)
    // Test based on old 'derivative' function
    f = problem1(x);
    g = numderivative(problem1, x, %eps^(1/4), order=4);
endfunction
function [f, g, ind] = myoptfuncvariablestep(x, ind)
    f = problem1(x);
    g = numderivative(problem1, x, order=4);
endfunction

// Initial guess:
x0 = 1*ones(5, 1);

// Perform optimization:
[fopt, xopt] = optim(myoptfuncfixedstep, x0, "gc", iprint=-1);

assert_checkequal(fopt, 0);
assert_checkequal(xopt', 1:5);

// Perform optimization:
[fopt, xopt] = optim(myoptfuncvariablestep, x0, "gc", iprint=-1);

assert_checkequal(fopt, 0);
assert_checkequal(xopt', 1:5);
